@@ -39,11 +39,12 @@ const PREC = {
   EQUALITY: 5,
   CONJUNCTION: 4,
   DISJUNCTION: 3,
+  VAR_DECL: 3,
   SPREAD: 2,
   SIMPLE_USER_TYPE: 2,
-  VAR_DECL: 1,
   ASSIGNMENT: 1,
   BLOCK: 1,
+  ARGUMENTS: 1,
   LAMBDA_LITERAL: 0,
   RETURN_OR_THROW: 0,
   COMMENT: 0
@@ -51,7 +52,7 @@ const PREC = {
 const DEC_DIGITS = token(sep1(/[0-9]+/, /_+/));
 const HEX_DIGITS = token(sep1(/[0-9a-fA-F]+/, /_+/));
 const BIN_DIGITS = token(sep1(/[01]/, /_+/));
-const REAL_EXPONENT = token(seq(/[eE]/, optional(/[+-]/), DEC_DIGITS))
+const REAL_EXPONENT = token(seq(/[eE]/, optional(/[+-]/), DEC_DIGITS));
 
 module.exports = grammar({
   name: "kotlin",
@@ -60,14 +61,6 @@ module.exports = grammar({
     // Ambiguous when used in an explicit delegation expression,
     // since the '{' could either be interpreted as the class body
     // or as the anonymous function body. Consider the following sequence:
-    //
-    // 'class'  simple_identifier  ':'  user_type  'by'  'fun'  '('  ')'  •  '{'  …
-    //
-    // Possible interpretations:
-    //
-    // 'class'  simple_identifier  ':'  user_type  'by'  (anonymous_function  'fun'  '('  ')'  •  function_body)
-    // 'class'  simple_identifier  ':'  user_type  'by'  (anonymous_function  'fun'  '('  ')')  •  '{'  …
-    [$.anonymous_function],
 
     // Member access operator '::' conflicts with callable reference
     [$._primary_expression, $.callable_reference],
@@ -84,7 +77,6 @@ module.exports = grammar({
     [$._postfix_unary_expression, $._expression],
 
     // ambiguity between generics and comparison operations (foo < b > c)
-    [$.call_expression, $.prefix_expression, $.comparison_expression],
     [$.call_expression, $.range_expression, $.comparison_expression],
     [$.call_expression, $.elvis_expression, $.comparison_expression],
     [$.call_expression, $.check_expression, $.comparison_expression],
@@ -102,7 +94,7 @@ module.exports = grammar({
     // ambiguity between multiple user types and class property/function declarations
     [$.user_type],
     [$.user_type, $.anonymous_function],
-    [$.user_type, $.function_type],
+    //[$.user_type, $.function_type],
 
     // ambiguity between annotated_lambda with modifiers and modifiers from var declarations
     [$.annotated_lambda, $.modifiers],
@@ -113,16 +105,29 @@ module.exports = grammar({
 
     // ambiguity between parameter modifiers in anonymous functions
     [$.parameter_modifiers, $._type_modifier],
+
+    // ambiguity between type modifiers before an @
+    [$.type_modifiers],
+    // ambiguity between associating type modifiers
+    [$.not_nullable_type],
+
+    [$.receiver_type],
+    [$.receiver_type, $._type],
   ],
 
   externals: $ => [
     $._automatic_semicolon,
     $._import_list_delimiter,
     $.safe_nav,
+    $.multiline_comment,
+    $._string_start,
+    $._string_end,
+    $.string_content,
   ],
 
   extras: $ => [
-    $.comment,
+    $.line_comment,
+    $.multiline_comment,
     /\s+/ // Whitespace
   ],
 
@@ -166,11 +171,13 @@ module.exports = grammar({
 
     import_header: $ => seq(
       "import",
-      $.identifier,
-      optional(choice(seq(".*"), $.import_alias)),
+      alias($._import_identifier, $.identifier),
+      optional(choice(seq(".", $.wildcard_import), $.import_alias)),
       $._semi
     ),
 
+    wildcard_import: _ => token.immediate("*"),
+
     import_alias: $ => seq("as", alias($.simple_identifier, $.type_identifier)),
 
     top_level_object: $ => seq($._declaration, optional($._semi)),
@@ -179,6 +186,7 @@ module.exports = grammar({
       optional($.modifiers),
       "typealias",
       alias($.simple_identifier, $.type_identifier),
+      optional($.type_parameters),
       "=",
       $._type
     ),
@@ -241,9 +249,11 @@ module.exports = grammar({
       ")"
     ),
 
+    binding_pattern_kind: $ => choice("val", "var"),
+
     class_parameter: $ => seq(
       optional($.modifiers),
-      optional(choice("val", "var")),
+      optional($.binding_pattern_kind),
       $.simple_identifier,
       ":",
       $._type,
@@ -317,7 +327,7 @@ module.exports = grammar({
       optional($.class_body)
     ),
 
-    _function_value_parameters: $ => seq(
+    function_value_parameters: $ => seq(
       "(",
       optional(sep1($._function_value_parameter, ",")),
       optional(","),
@@ -330,12 +340,12 @@ module.exports = grammar({
       optional(seq("=", $._expression))
     ),
 
-    _receiver_type: $ => seq(
+    receiver_type: $ => seq(
       optional($.type_modifiers),
       choice (
-        $._type_reference,
         $.parenthesized_type,
-        $.nullable_type
+        $.nullable_type,
+        $._type_reference,
       )
     ),
 
@@ -343,9 +353,9 @@ module.exports = grammar({
       optional($.modifiers),
       "fun",
       optional($.type_parameters),
-      optional(seq($._receiver_type, optional('.'))),
+      optional(seq(field("receiver", $.receiver_type), optional('.'))),
       $.simple_identifier,
-      $._function_value_parameters,
+      $.function_value_parameters,
       optional(seq(":", $._type)),
       optional($.type_constraints),
       optional($.function_body)
@@ -361,9 +371,9 @@ module.exports = grammar({
 
     property_declaration: $ => prec.right(seq(
       optional($.modifiers),
-      choice("val", "var"),
+      $.binding_pattern_kind,
       optional($.type_parameters),
-      optional(seq($._receiver_type, optional('.'))),
+      optional(seq(field("receiver", $.receiver_type), optional('.'))),
       choice($.variable_declaration, $.multi_variable_declaration),
       optional($.type_constraints),
       optional(choice(
@@ -423,7 +433,7 @@ module.exports = grammar({
     secondary_constructor: $ => seq(
       optional($.modifiers),
       "constructor",
-      $._function_value_parameters,
+      $.function_value_parameters,
       optional(seq(":", $.constructor_delegation_call)),
       optional($._block)
     ),
@@ -457,16 +467,25 @@ module.exports = grammar({
     _type: $ => seq(
       optional($.type_modifiers),
       choice(
+        $.function_type,
         $.parenthesized_type,
         $.nullable_type,
         $._type_reference,
-        $.function_type
+        $.not_nullable_type
       )
     ),
 
-    _type_reference: $ => choice(
+    _type_reference: $ => prec.left(1, choice(
       $.user_type,
       "dynamic"
+    )),
+
+    not_nullable_type: $ => seq(
+      optional($.type_modifiers),
+      choice($.user_type, $.parenthesized_user_type),
+      '&',
+      optional($.type_modifiers),
+      choice($.user_type, $.parenthesized_user_type),
     ),
 
     nullable_type: $ => seq(
@@ -497,7 +516,7 @@ module.exports = grammar({
     _type_projection_modifier: $ => $.variance_modifier,
 
     function_type: $ => seq(
-      optional(seq($._simple_user_type, ".")), // TODO: Support "real" types
+      optional(seq(field("receiver", $.receiver_type), ".")),
       $.function_type_parameters,
       "->",
       $._type
@@ -607,7 +626,6 @@ module.exports = grammar({
 
     _unary_expression: $ => choice(
       $.postfix_expression,
-      $.call_expression,
       $.indexing_expression,
       $.navigation_expression,
       $.prefix_expression,
@@ -685,7 +703,7 @@ module.exports = grammar({
       // this introduces ambiguities with 'less than' for comparisons
       optional($.type_arguments),
       choice(
-        seq(optional($.value_arguments), $.annotated_lambda),
+        prec(PREC.ARGUMENTS, seq(optional($.value_arguments), $.annotated_lambda)),
         $.value_arguments
       )
     )),
@@ -698,7 +716,16 @@ module.exports = grammar({
 
     type_arguments: $ => seq("<", sep1($.type_projection, ","), ">"),
 
-    value_arguments: $ => seq("(", optional(sep1($.value_argument, ",")), ")"),
+    value_arguments: $ => seq(
+      "(",
+      optional(
+        seq(
+          sep1($.value_argument, ","),
+          optional(","),
+        )
+      ),
+      ")"
+    ),
 
     value_argument: $ => seq(
       optional($.annotation),
@@ -711,8 +738,9 @@ module.exports = grammar({
       $.parenthesized_expression,
       $.simple_identifier,
       $._literal_constant,
-      $._string_literal,
+      $.string_literal,
       $.callable_reference,
+      $.call_expression,
       $._function_literal,
       $.object_literal,
       $.collection_literal,
@@ -726,7 +754,13 @@ module.exports = grammar({
 
     parenthesized_expression: $ => seq("(", $._expression, ")"),
 
-    collection_literal: $ => seq("[", $._expression, repeat(seq(",", $._expression)), "]"),
+    // https://kotlinlang.org/spec/syntax-and-grammar.html#grammar-rule-collectionLiteral
+    collection_literal: $ => seq(
+      "[",
+      $._expression,
+      repeat(seq(",", $._expression)),
+      optional(","),
+      "]"),
 
     _literal_constant: $ => choice(
       $.boolean_literal,
@@ -735,36 +769,19 @@ module.exports = grammar({
       $.bin_literal,
       $.character_literal,
       $.real_literal,
-      "null",
+      $.null_literal,
       $.long_literal,
       $.unsigned_literal
     ),
 
-    _string_literal: $ => choice(
-      $.line_string_literal,
-      $.multi_line_string_literal
-    ),
-
-    line_string_literal: $ => seq('"', repeat(choice($._line_string_content, $._interpolation)), '"'),
-
-    multi_line_string_literal: $ => seq(
-      '"""',
-      repeat(choice(
-        $._multi_line_string_content,
-        $._interpolation
-      )),
-      '"""'
-    ),
-
-    _line_string_content: $ => choice(
-      $._line_str_text,
-      $.character_escape_seq
+    string_literal: $ => seq(
+      $._string_start,
+      repeat(choice($.string_content, $._interpolation)),
+      $._string_end,
     ),
 
     line_string_expression: $ => seq("${", $._expression, "}"),
 
-    _multi_line_string_content: $ => choice($._multi_line_str_text, '"'),
-
     _interpolation: $ => choice(
       seq("${", alias($._expression, $.interpolated_expression), "}"),
       seq("$", alias($.simple_identifier, $.interpolated_identifier))
@@ -793,7 +810,7 @@ module.exports = grammar({
     anonymous_function: $ => prec.right(seq(
       "fun",
       optional(seq(sep1($._simple_user_type, "."), ".")), // TODO
-      $._function_value_parameters,
+      $.function_value_parameters,
       optional(seq(":", $._type)),
       optional($.function_body)
     )),
@@ -809,26 +826,29 @@ module.exports = grammar({
       $.class_body
     ),
 
-    this_expression: $ => "this",
-
-    super_expression: $ => seq(
-      "super",
-      // TODO optional(seq("<", $._type, ">")),
-      // TODO optional(seq("@", $.simple_identifier))
+    this_expression: $ => choice(
+      "this",
+      $._this_at
     ),
 
+    super_expression: $ => prec.right(choice(
+      "super",
+      seq("super", "<", $._type, ">"),
+      $._super_at
+    )),
+
     if_expression: $ => prec.right(seq(
       "if",
-      "(", $._expression, ")",
+      "(", field('condition', $._expression), ")",
       choice(
-        $.control_structure_body,
-        ";",
+        field('consequence', $.control_structure_body),
         seq(
-          optional($.control_structure_body),
+          optional(field('consequence', $.control_structure_body)),
           optional(";"),
           "else",
-          choice($.control_structure_body, ";")
-        )
+          choice(field('alternative', $.control_structure_body), ";")
+        ),
+        ";"
       )
     )),
 
@@ -983,7 +1003,8 @@ module.exports = grammar({
       "sealed",
       "annotation",
       "data",
-      "inner"
+      "inner",
+      "value",
     ),
 
     member_modifier: $ => choice(
@@ -1083,6 +1104,7 @@ module.exports = grammar({
       "expect",
       "data",
       "inner",
+      "value",
       "actual",
       "set",
       "get"
@@ -1091,6 +1113,13 @@ module.exports = grammar({
 
     identifier: $ => sep1($.simple_identifier, "."),
 
+    // Adapted from tree-sitter-java, helps to avoid a conflic with
+    // wildcard_import node while being compatible with identifier
+    _import_identifier: $ => choice(
+      $.simple_identifier,
+      seq($._import_identifier, ".", $.simple_identifier),
+    ),
+
     // ====================
     // Lexical grammar
     // ====================
@@ -1100,11 +1129,7 @@ module.exports = grammar({
     // General
     // ==========
 
-    // Source: https://github.com/tree-sitter/tree-sitter-java/blob/bc7124d924723e933b6ffeb5f22c4cf5248416b7/grammar.js#L1030
-    comment: $ => token(prec(PREC.COMMENT, choice(
-      seq("//", /.*/),
-      seq("/*", /[^*]*\*+([^/*][^*]*\*+)*/, "/")
-    ))),
+    line_comment: $ => token(seq('//', /.*/)),
 
     // ==========
     // Separators and operations
@@ -1115,15 +1140,38 @@ module.exports = grammar({
     // Keywords
     // ==========
 
-    _return_at: $ => seq("return@", $._lexical_identifier),
+    _return_at: $ => seq(
+      "return@",
+      alias($._lexical_identifier, $.label)
+    ),
 
-    _continue_at: $ => seq("continue@", $._lexical_identifier),
+    _continue_at: $ => seq(
+      "continue@",
+      alias($._lexical_identifier, $.label)
+    ),
 
-    _break_at: $ => seq("break@", $._lexical_identifier),
+    _break_at: $ => seq(
+      "break@",
+      alias($._lexical_identifier, $.label)
+    ),
 
-    _this_at: $ => seq("this@", $._lexical_identifier),
+    _this_at: $ => seq(
+      "this@",
+      alias($._lexical_identifier, $.type_identifier)
+    ),
 
-    _super_at: $ => seq("super@", $._lexical_identifier),
+    _super_at: $ => choice(
+      seq(
+        "super@",
+        alias($._lexical_identifier, $.type_identifier)
+      ),
+      seq(
+        "super",
+        "<", $._type, ">",
+        token.immediate("@"),
+        alias($._lexical_identifier, $.type_identifier)
+      )
+    ),
 
     // ==========
     // Literals
@@ -1148,8 +1196,7 @@ module.exports = grammar({
 
     unsigned_literal: $ => seq(
       choice($.integer_literal, $.hex_literal, $.bin_literal),
-      /[uU]/,
-      optional("L")
+      /[uU]L?/
     ),
 
     long_literal: $ => seq(
@@ -1170,6 +1217,8 @@ module.exports = grammar({
       $._escaped_identifier
     ),
 
+    null_literal: $ => "null",
+
     // ==========
     // Identifiers
     // ==========
@@ -1179,7 +1228,7 @@ module.exports = grammar({
       $._backtick_identifier,
     ),
 
-    _alpha_identifier: $ => /[a-zA-Z_][a-zA-Z_0-9]*/,
+    _alpha_identifier: $ => /[\p{L}_][\p{L}_\p{Nd}]*/,
 
     _backtick_identifier: $ => /`[^\r\n`]+`/,
 
@@ -1190,13 +1239,6 @@ module.exports = grammar({
 
     _escaped_identifier: $ => /\\[tbrn'"\\$]/,
 
-    // ==========
-    // Strings
-    // ==========
-
-    _line_str_text: $ => /[^\\"$]+/,
-
-    _multi_line_str_text: $ => /[^"$]+/
   }
 });
 
