@@ -50,6 +50,7 @@ module.exports = grammar({
     $.float,
     $.block_comment,
     '{',
+    '"',
   ],
 
   extras: $ => [
@@ -80,6 +81,7 @@ module.exports = grammar({
     tagged_block: $ => seq($.tag, $.block),
 
     declaration: $ => choice(
+      $.build_tag,
       $.package_declaration,
       $.import_declaration,
       $.procedure_declaration,
@@ -97,6 +99,8 @@ module.exports = grammar({
       $._expression_no_tag,
     ),
 
+    build_tag: $ => seq('#+', /.+/),
+
     package_declaration: $ => seq('package', $.identifier),
 
     import_declaration: $ => seq(
@@ -104,7 +108,15 @@ module.exports = grammar({
       optional('foreign'),
       'import',
       optional(field('alias', $.identifier)),
-      choice($.string, seq('{', commaSep1($.string), optional(','), '}')),
+      choice(
+        $.string,
+        seq(
+          '{',
+          commaSep1(choice($.string, $.identifier)),
+          optional(','),
+          '}',
+        ),
+      ),
     ),
 
     procedure_declaration: $ => seq(
@@ -265,7 +277,7 @@ module.exports = grammar({
         $.identifier,
         seq(
           '(',
-          commaSep1(seq($.identifier, optional(seq('=', choice($.literal, $.identifier))))),
+          commaSep1(seq($.identifier, optional(seq('=', $.expression)))),
           ')',
         ),
       ),
@@ -280,22 +292,33 @@ module.exports = grammar({
       ')',
     ),
     parameter: $ => prec.right(seq(
+      commaSep1($._param_header),
+      optional($._param_type),
+    )),
+    _param_header: $ => seq(
       optional($.tag),
       optional('using'),
-      commaSep1(seq(
-        optional('$'),
-        choice(
-          $.identifier,
-          $.variadic_type,
-          $.array_type,
-          $.pointer_type,
-          $.field_type,
-          $._procedure_type,
-        ),
-      )),
-      optional(seq(':', optional($.tag), $.type, optional($.identifier), optional(seq('=', $.expression)))),
-    )),
+      optional('$'),
+      choice(
+        $.identifier,
+        $.variadic_type,
+        $.array_type,
+        $.pointer_type,
+        $.field_type,
+        $._procedure_type,
+      ),
+    ),
+    _param_type: $ => seq(
+      ':',
+      optional($.tag),
+      $.type,
+      optional($.identifier),
+      optional(seq('=', $.expression)),
+    ),
+
+
     default_parameter: $ => seq(
+      optional($.tag),
       optional('using'),
       $.identifier,
       ':=',
@@ -371,7 +394,10 @@ module.exports = grammar({
       )),
       optional($.tag),
       field('condition', $.expression),
-      field('consequence', choice($.block, seq('do', $.statement))),
+      choice(
+        field('consequence', $.block),
+        seq('do', field('consequence', $.statement)),
+      ),
       repeat($.else_if_clause),
       optional($.else_clause),
     )),
@@ -384,18 +410,24 @@ module.exports = grammar({
         ';',
       )),
       field('condition', $.expression),
-      field('consequence', choice($.block)),
+      choice(
+        field('consequence', $.block),
+        seq('do', field('consequence', $.statement)),
+      ),
     ),
 
     else_clause: $ => seq(
       'else',
-      field('consequence', $.block),
+      choice(
+        field('consequence', $.block),
+        seq('do', field('consequence', $.statement)),
+      ),
     ),
 
     when_statement: $ => prec.right(seq(
       'when',
       $.expression,
-      $.block,
+      choice($.block, seq('do', $.statement)),
       repeat($.else_when_clause),
       optional($.else_clause),
     )),
@@ -761,6 +793,7 @@ module.exports = grammar({
 
     enum_type: $ => seq(
       'enum',
+      optional(field('underlying_type', $.type)),
       '{',
       commaSep1(seq($.identifier, optional(seq('=', $.expression)))),
       optional(','),
@@ -776,7 +809,7 @@ module.exports = grammar({
       '}',
     ),
 
-    named_type: $ => prec.right(seq($.identifier, ':', $.type, optional(seq('=', $.literal)))),
+    named_type: $ => prec.right(seq(commaSep1($.identifier), ':', $.type, optional(seq('=', $.literal)))),
 
     default_type: $ => seq($.identifier, ':=', $.expression),
 
@@ -830,7 +863,7 @@ module.exports = grammar({
       $.type,
       '{',
       optional(seq(
-        commaSep1(seq($.expression, '=', $.literal)),
+        commaSep1(seq($.expression, '=', $.expression)),
         optional(','),
       )),
       '}',
@@ -840,6 +873,10 @@ module.exports = grammar({
       'bit_set',
       '[',
       $.expression,
+      optional(seq(
+        ';',
+        field('underlying_type', $.type),
+      )),
       ']',
       '{',
       commaSep($.expression),
@@ -940,7 +977,7 @@ module.exports = grammar({
 
     tag: _ => token(seq(/#[a-zA-Z_][a-zA-Z0-9_]*/, optional(seq('(', /\w*/, ')')))),
 
-    identifier: _ => /[_\p{XID_Start}][_\p{XID_Continue}]*/,
+    identifier: _ => /[_\p{XID_Start}][_\p{XID_Continue}]*/u,
 
     field_identifier: $ => prec(-1, seq($.identifier, repeat1(seq('.', $.identifier)))),
 
@@ -966,8 +1003,7 @@ module.exports.PREC = PREC;
  *
  * @param {Rule} rule
  *
- * @return {ChoiceRule}
- *
+ * @returns {ChoiceRule}
  */
 function commaSep(rule) {
   return optional(commaSep1(rule));
@@ -978,8 +1014,7 @@ function commaSep(rule) {
  *
  * @param {Rule} rule
  *
- * @return {SeqRule}
- *
+ * @returns {SeqRule}
  */
 function commaSep1(rule) {
   return sep1(rule, ',');
@@ -992,8 +1027,7 @@ function commaSep1(rule) {
  *
  * @param {GrammarSymbols<any>} $
  *
- * @return {SeqRule}
- *
+ * @returns {SeqRule}
  */
 function commaExternalSep1(rule, $) {
   return sep1(rule, choice(',', alias($._nl_comma, ',')));
@@ -1002,12 +1036,11 @@ function commaExternalSep1(rule, $) {
 /**
  * Creates a rule to match zero or more occurrences of `rule` separated by `sep`
  *
- * @param {RegExp|Rule|String} rule
+ * @param {RegExp | Rule | string} rule
  *
- * @param {RegExp|Rule|String} sep
- *
- * @return {ChoiceRule}
+ * @param {RegExp | Rule | string} sep
  *
+ * @returns {ChoiceRule}
  */
 function sep(rule, sep) {
   return optional(seq(rule, repeat(seq(sep, optional(rule)))));
@@ -1016,12 +1049,11 @@ function sep(rule, sep) {
 /**
  * Creates a rule to match one or more occurrences of `rule` separated by `sep`
  *
- * @param {RegExp|Rule|String} rule
+ * @param {RegExp | Rule | string} rule
  *
- * @param {RegExp|Rule|String} sep
- *
- * @return {SeqRule}
+ * @param {RegExp | Rule | string} sep
  *
+ * @returns {SeqRule}
  */
 function sep1(rule, sep) {
   return seq(rule, repeat(seq(sep, rule)));
