@@ -1,5 +1,10 @@
 (ERROR) @error
-(comment) @comment
+
+[
+ (line_comment)
+ (block_comment)
+ (shebang)
+] @comment
 
 (identifier) @variable
 (import_path) @variable
@@ -12,8 +17,10 @@
   receiver: (receiver)
   name: (identifier) @method)
 
+(short_lambda
+  (reference_expression) @parameter)
 (call_expression
- name: (selector_expression
+  name: (selector_expression
     field: (reference_expression) @method))
 
 (type_reference_expression) @type
